const {User} = require('../../../../dao/models');
const Joi = require('joi');
const ApplicationError = require('../../../../utils/application-error');
/**
* @module An object representing the Post HTTP method for creating a user.
* @type {Object}
* @property {string} method - HTTP method (post).
* @property {string} path - The path to process the request ("/register").
* @property {Joi.ObjectSchema} validationSchema - Input validation scheme.
* @property {Array} middlewares - Middlewares that can be applied to the request processing (empty array).
* @property {Function} handler - The request handler function for creating a user.
* @async
* @param {Object} req - Request object.
* @param {Object} res - Response object.
* @throws {Error} - Exception if an error occurred during creation.
*/
module.exports = {
  method: 'post',
  path: '/register',
  validationSchema: Joi.object({
    email: Joi.string().required(),
    password: Joi.string().required(),
  }),
  middlewares: [],
  handler: async function (req, res) {
    const oldUser = await User.findOne({where: {email: req.body.email}});
    if (oldUser) {
      throw ApplicationError.NotUnique();
    }

    req.body.passwordHashed = req.body.password;
    delete req.body.password;

    const user = await User.create(req.body);
    return await user.getTokens();
  }
};
